@@ -38,29 +38,7 @@
 
 #include "credentials.h"
 #include "sslutils.h"
-
-int
-globus(int version)
-{
-  if (version == 0) {
-    char *gver = getenv("GLOBUS_VERSION");
-
-    if (gver) {
-      char *tmp;
-      
-      version = strtol(gver, &tmp, 10);
-      if (!(*tmp))
-        return 22;
-    }
-  }
-  
-  if (version >= 42 || version == 0)
-    version = 22;
-
-  return version;
-}
-
-
+#include "voms_cert_type.h"
 
 X509 *
 get_real_cert(X509 *base, STACK_OF(X509) *stk)
@@ -68,15 +46,32 @@ get_real_cert(X509 *base, STACK_OF(X509) *stk)
   X509 *cert = NULL;
   int i;
 
-  if (!proxy_check_proxy_name(base))
+  voms_cert_type_t cert_type;
+
+
+  if (voms_get_cert_type(base, &cert_type)){
+    // FIXME: This is just for backward compatibility, where error in the
+    // proxy_check_proxy_name call weren't handled
+    return base;
+  }
+
+  if (!VOMS_IS_PROXY(cert_type)){
     return base;
 
+  }
   int num_certs = sk_X509_num(stk);
 
   /* Determine id data */
   for (i = 0; i < num_certs; i++) {
     cert = sk_X509_value(stk, i);
-    if (!proxy_check_proxy_name(cert)) {
+
+    if (voms_get_cert_type(cert, &cert_type)){
+    // FIXME: This is just for backward compatibility, where error in the
+    // proxy_check_proxy_name call weren't handled
+      return cert;
+    }
+
+    if (!VOMS_IS_PROXY(cert_type)){
       return cert;
     }
   }
