@@ -42,8 +42,6 @@
 #ifndef VOMS_GSISOCKETCLIENT
 #define VOMS_GSISOCKETCLIENT
 
-
-
 /** This super class header file. */
 #include <openssl/evp.h>
 #include <openssl/x509.h>
@@ -53,8 +51,6 @@
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
 
 extern "C" {
 #include "sslutils.h"
@@ -76,40 +72,25 @@ public:
    * @param p the secure server port.
    * @param b the backlog, that is the maximum number of outstanding connection requests.
    */
-  GSISocketClient(const std::string&, int, int, void* = NULL);
+  GSISocketClient(const std::string&, int);
   /**
    * Destructor.
    */  
-  virtual ~GSISocketClient();
-
-  void RedirectGSIOutput(FILE *fp) { gsi_logfile = fp; }
-  /**
-   * Set the server contact. 
-   * @param contact the server contact string to set.
-   */
-  void ServerContact(const std::string& contact) { _server_contact = contact; }
-  /**
-   * Sets required connection flags.
-   * @param flags is a bitwise or of all the flags required.
-   */
-  //  void SetFlags(int flags);
-
-
-  //  void SetLogger(void *l);
+  ~GSISocketClient();
 
   /**
    * Open the connection.
    * @return true for successful opening, false otherwise.
    */
-  virtual bool Open();
+  bool Open();
   /**
    * Close the connection.
    * @return true for successful close, false otehrwise.
    */
-  virtual void Close();
+  void Close();
 
-  virtual bool post_connection_check(SSL*);
-  virtual bool LoadCredentials(const char *, X509 *, STACK_OF(X509) *, EVP_PKEY *);
+  bool post_connection_check(SSL*);
+  bool LoadCredentials(const char *, X509 *, STACK_OF(X509) *, EVP_PKEY *);
 
 protected:
   /**
@@ -123,39 +104,24 @@ protected:
 private:
   std::string host;
   int port;
-  int version;
-   /** The Secure Shell context identifier. */
-  gss_ctx_id_t context; 
-  gss_cred_id_t credential;
-  std::string _server_contact;
-   //bool _do_mutual_authentication;
-  //  int conflags;
-  FILE *gsi_logfile;
+
   bool opened;
   int sck;
 
 public:
   std::string     own_subject;
-  std::string     own_ca;
   EVP_PKEY       *upkey;
   X509           *ucert;
   STACK_OF(X509) *cert_chain;
   char           *cacertdir;
-  std::string     peer_subject;
-  std::string     peer_ca;
-  EVP_PKEY       *peer_key;
-  X509           *peer_cert;
-  //  void           *logh;
   SSL *ssl;
   SSL_CTX *ctx;
   BIO *conn;
-  void *pvd;
 
   bool Send(const std::string &s);
   bool Receive(std::string &s);
 
 private:
-  struct sockaddr_in peeraddr_in;	/**< Address for peer socket.*/
   std::string error;
   void SetError(const std::string&);
   void SetErrorGlobus(const std::string&, int, int, int);
