@@ -39,7 +39,6 @@ extern void       *LogAddStreamer(void *, const char *,
                                   int (*)(void *, const char *), 
                                   void (*)(void *),
                                   void (*)(void *, const char *, const char*));
-extern void        StartLogger(void *, int);
 extern void        LogDestroy(void *);
 extern loglevels   LogLevel(void *, loglevels);
 extern logtypes    LogType(void *, int);
@@ -48,13 +47,12 @@ extern const char *LogService(void *, const char *);
 extern const char *LogFormat(void *, const char *);
 extern int         LogMessage(void *, loglevels, logtypes, const char *, const char *, int, const char *);
 extern int         LogMessageF(const char *, int, const char *, void *, loglevels, logtypes, const char *, ...);
-extern int         LogBuffer(FILE *, void *, loglevels, logtypes, const char *);
 extern logtypes    SetCurLogType(void *, logtypes);
 extern void        LogActivate(void *, const char *);
 extern void        LogDeactivate(void *, const char *);
 extern void        LogOption(void *, const char *, const char *);
 extern void        LogOptionInt(void *, const char *, int);
-extern void        SetOwner(pid_t);
+extern int         LogLevelMin(void *, loglevels);
 
 #define LOG(h, lev, type, str) \
 LogMessage((h), (lev), (type), (str), FUNC_NAME, __LINE__, __FILE__)
