@@ -1,6 +1,7 @@
 #include "sslutils.h"
-#include "openssl/x509_vfy.h"
-#include "openssl/x509v3.h"
+#include <openssl/x509_vfy.h>
+#include <openssl/x509v3.h>
+#include "ssl_compat.h"
 
 #include <iostream>
 #include <cstdlib>
@@ -63,7 +64,6 @@ err:
 
 int verify_cert(X509_STORE_CTX *ctx) {
 
-  ctx->check_issued = proxy_check_issued;
   return X509_verify_cert(ctx);
 }
 
@@ -74,8 +74,6 @@ proxy_verify_desc *setup_initializers(const char *cadir)
 
   pvd  = (proxy_verify_desc*)      malloc(sizeof(proxy_verify_desc));
   pvxd = (proxy_verify_ctx_desc *) malloc(sizeof(proxy_verify_ctx_desc));
-  pvd->cert_store = NULL;
-
 
   if (!pvd || !pvxd) {
     free(pvd);
@@ -183,9 +181,7 @@ int main(int argc, char* argv[]){
     internal_error("Error creating X.509 store");
   }
 
-  if (!X509_STORE_set_verify_cb_func(store, proxy_verify_callback)){
-    internal_error("Error setting context store certificate verify callback");
-  }
+  X509_STORE_set_verify_cb(store, proxy_verify_callback);
 
   if (!(lookup = X509_STORE_add_lookup(store, X509_LOOKUP_hash_dir()))){
     internal_error("Error creating store CA dir lookup");
@@ -203,6 +199,7 @@ int main(int argc, char* argv[]){
     internal_error("Error creating X509_STORE_CTX object");
   }
 
+  X509_STORE_set_check_issued(store, proxy_check_issued);
   if (X509_STORE_CTX_init(ctx, store, cert, cert_chain) != 1) {
     internal_error("Error initializing verification context");
   }
@@ -218,7 +215,7 @@ int main(int argc, char* argv[]){
     exit(-1);
   }
 
-  printf("Certificate chain verified succesfully\n");
+  printf("Certificate chain verified successfully\n");
 
   return 0;
 }
